%% Example code for the application of degree-normalization to improve FC fingerprinting
%
%  From the following publication (please cite us!) : 
%       Chiem B, Abbas K, Amico E, Duong-Tran DA, Crevecoeur F, and Goñi J. 
%       Improving Functional Connectome Fingerprinting with Degree-Normalization. 
%       Brain connectivity, 2021 (in press).
%      https://doi.org/10.1089/brain.2020.0968
%
%  This code is based on the code developed at:
%      The quest for identifiability in human functional connectomes (2018) Amico E, Goñi J. Scientific Reports 8.1: 8254
%      https://doi.org/10.1038/s41598-018-25089-1
%
%  The example considers three fingerprinting metrics : 
%    - Differential identifiability (Idiff)
%    - Identification rate (ID_rate)
%    - Matching rate (M_rate)
%
%  Run this code in the folder containing the data file
%  (FC_test_retest.mat) and the functions f_PCA_identifiability.m,
%  f_compute_id_metrics.m and f_plot_results.m

% initialize environment
close all;
clearvars
clc;

%% Load data
%  Sample of 40 test-retest pairs of resting-state FCs
%  Brain atlas : HCP MMP1.0 (Glasser) parcellation, 374 regions
%  FC_test_retest(:,:,i) and FC_test_retest(:,:,i+40) for the test-retest
%  pair of FCs (374 x 374 matrices) of subject i

disp('Loading data ...');
load('FC_test_retest.mat');
disp('Data loaded !')

nROIs = size(FC_test_retest,1);
nFCs  = size(FC_test_retest,3);
nSubj = nFCs/2; % Two FCs per subject

%% Separate modalities : Baseline, Absolute and Degree-normalized FCs

baseline_FCs   = FC_test_retest;
absolute_FCs   = zeros(size(FC_test_retest));
normalized_FCs = zeros(size(FC_test_retest));

disp('Separating modalities : baseline, absolute and normalized FCs ...')
for iFC = 1:nFCs
    myFC = squeeze(baseline_FCs(:,:,iFC)); % Original FC
    
    % Absolute FC
    absolute_FCs(:,:,iFC) = abs(myFC); % Entry-wise absolute value
    
    % Degree-normalized FC
    D = diag(sum(abs(myFC),2)); % Degree matrix (diagonal)
    normalized_FCs(:,:,iFC) = D^(-0.5)*abs(myFC)*D^(-0.5); % Degree-normalization
end
disp('Done !')

%% Configuration
%  For the call to the f_PCA_identifiability developed by Amico and Goni (2018)

configs.numRegions = nROIs;
configs.mask_ut = triu(true(nROIs,nROIs),1); % upper triangular mask
configs.numFCs = nFCs;
configs.numVisits = 2; % 2 visits per subject (test-retest)
configs.max_numPCs = configs.numFCs; % maximum number of PCs == data dimension

%% Index vectors for test and retest (visit 1 and visit 2; first half and second half)

Test_index = 1:0.5*configs.numFCs; % Here, first half is test, second half is retest
Retest_index = 0.5*configs.numFCs+1:configs.numFCs;

%% create 2D input matrix 

baseline_FCs_2D   = zeros(nnz(configs.mask_ut),configs.numFCs);
absolute_FCs_2D   = zeros(nnz(configs.mask_ut),configs.numFCs);
normalized_FCs_2D = zeros(nnz(configs.mask_ut),configs.numFCs);

disp('Creating data matrices ...')
for i=1:configs.numFCs
    temp_baseline   = squeeze(baseline_FCs(:,:,i));
    temp_absolute   = squeeze(absolute_FCs(:,:,i));
    temp_normalized = squeeze(normalized_FCs(:,:,i));
    
    baseline_FCs_2D(:,i)   = temp_baseline(configs.mask_ut);
    absolute_FCs_2D(:,i)   = temp_absolute(configs.mask_ut);
    normalized_FCs_2D(:,i) = temp_normalized(configs.mask_ut);
end
disp('Done !');

disp('Applying PCA decomposition ...')
res_baseline   = f_PCA_identifiability(baseline_FCs_2D,Test_index,Retest_index,configs);
disp('Baseline FCs : done !')
res_absolute   = f_PCA_identifiability(absolute_FCs_2D,Test_index,Retest_index,configs);
disp('Absolute FCs : done !')
res_normalized = f_PCA_identifiability(normalized_FCs_2D,Test_index,Retest_index,configs);
disp('Normalized FCs : done !')

%% Plot results
%  You can adapt the display settings in the function

disp('Plotting the results ...')
hFig = f_plot_results(res_baseline,res_absolute,res_normalized);